/****************************************************************************
** Meta object code from reading C++ file 'abstractpushprovider.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kunifiedpush-26.04.3/src/distributor/abstractpushprovider.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractpushprovider.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KUnifiedPush20AbstractPushProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto KUnifiedPush::AbstractPushProvider::qt_create_metaobjectdata<qt_meta_tag_ZN12KUnifiedPush20AbstractPushProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KUnifiedPush::AbstractPushProvider",
        "messageReceived",
        "",
        "KUnifiedPush::Message",
        "msg",
        "clientRegistered",
        "KUnifiedPush::Client",
        "client",
        "KUnifiedPush::AbstractPushProvider::Error",
        "error",
        "errorMsg",
        "clientUnregistered",
        "connected",
        "disconnected",
        "messageAcknowledged",
        "messageIdentifier",
        "urgencyChanged",
        "Error",
        "NoError",
        "ProviderRejected",
        "TransientNetworkError",
        "ActionRequired"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'messageReceived'
        QtMocHelpers::SignalData<void(const KUnifiedPush::Message &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'clientRegistered'
        QtMocHelpers::SignalData<void(const KUnifiedPush::Client &, KUnifiedPush::AbstractPushProvider::Error, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 }, { QMetaType::QString, 10 },
        }}),
        // Signal 'clientRegistered'
        QtMocHelpers::SignalData<void(const KUnifiedPush::Client &, KUnifiedPush::AbstractPushProvider::Error)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'clientRegistered'
        QtMocHelpers::SignalData<void(const KUnifiedPush::Client &)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'clientUnregistered'
        QtMocHelpers::SignalData<void(const KUnifiedPush::Client &, KUnifiedPush::AbstractPushProvider::Error)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'clientUnregistered'
        QtMocHelpers::SignalData<void(const KUnifiedPush::Client &)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void(KUnifiedPush::AbstractPushProvider::Error, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::QString, 10 },
        }}),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void(KUnifiedPush::AbstractPushProvider::Error)>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'messageAcknowledged'
        QtMocHelpers::SignalData<void(const KUnifiedPush::Client &, const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::QString, 15 },
        }}),
        // Signal 'urgencyChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(17, 17, QMC::EnumFlags{}).add({
            {   18, Error::NoError },
            {   19, Error::ProviderRejected },
            {   20, Error::TransientNetworkError },
            {   21, Error::ActionRequired },
        }),
    };
    return QtMocHelpers::metaObjectData<AbstractPushProvider, qt_meta_tag_ZN12KUnifiedPush20AbstractPushProviderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KUnifiedPush::AbstractPushProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KUnifiedPush20AbstractPushProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KUnifiedPush20AbstractPushProviderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KUnifiedPush20AbstractPushProviderE_t>.metaTypes,
    nullptr
} };

void KUnifiedPush::AbstractPushProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AbstractPushProvider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->messageReceived((*reinterpret_cast<std::add_pointer_t<KUnifiedPush::Message>>(_a[1]))); break;
        case 1: _t->clientRegistered((*reinterpret_cast<std::add_pointer_t<KUnifiedPush::Client>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KUnifiedPush::AbstractPushProvider::Error>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 2: _t->clientRegistered((*reinterpret_cast<std::add_pointer_t<KUnifiedPush::Client>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KUnifiedPush::AbstractPushProvider::Error>>(_a[2]))); break;
        case 3: _t->clientRegistered((*reinterpret_cast<std::add_pointer_t<KUnifiedPush::Client>>(_a[1]))); break;
        case 4: _t->clientUnregistered((*reinterpret_cast<std::add_pointer_t<KUnifiedPush::Client>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KUnifiedPush::AbstractPushProvider::Error>>(_a[2]))); break;
        case 5: _t->clientUnregistered((*reinterpret_cast<std::add_pointer_t<KUnifiedPush::Client>>(_a[1]))); break;
        case 6: _t->connected(); break;
        case 7: _t->disconnected((*reinterpret_cast<std::add_pointer_t<KUnifiedPush::AbstractPushProvider::Error>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->disconnected((*reinterpret_cast<std::add_pointer_t<KUnifiedPush::AbstractPushProvider::Error>>(_a[1]))); break;
        case 9: _t->messageAcknowledged((*reinterpret_cast<std::add_pointer_t<KUnifiedPush::Client>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->urgencyChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AbstractPushProvider::*)(const KUnifiedPush::Message & )>(_a, &AbstractPushProvider::messageReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractPushProvider::*)(const KUnifiedPush::Client & , KUnifiedPush::AbstractPushProvider::Error , const QString & )>(_a, &AbstractPushProvider::clientRegistered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractPushProvider::*)(const KUnifiedPush::Client & , KUnifiedPush::AbstractPushProvider::Error )>(_a, &AbstractPushProvider::clientUnregistered, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractPushProvider::*)()>(_a, &AbstractPushProvider::connected, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractPushProvider::*)(KUnifiedPush::AbstractPushProvider::Error , const QString & )>(_a, &AbstractPushProvider::disconnected, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractPushProvider::*)(const KUnifiedPush::Client & , const QString & )>(_a, &AbstractPushProvider::messageAcknowledged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractPushProvider::*)()>(_a, &AbstractPushProvider::urgencyChanged, 10))
            return;
    }
}

const QMetaObject *KUnifiedPush::AbstractPushProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KUnifiedPush::AbstractPushProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KUnifiedPush20AbstractPushProviderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KUnifiedPush::AbstractPushProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KUnifiedPush::AbstractPushProvider::messageReceived(const KUnifiedPush::Message & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KUnifiedPush::AbstractPushProvider::clientRegistered(const KUnifiedPush::Client & _t1, KUnifiedPush::AbstractPushProvider::Error _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void KUnifiedPush::AbstractPushProvider::clientUnregistered(const KUnifiedPush::Client & _t1, KUnifiedPush::AbstractPushProvider::Error _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 6
void KUnifiedPush::AbstractPushProvider::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KUnifiedPush::AbstractPushProvider::disconnected(KUnifiedPush::AbstractPushProvider::Error _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 9
void KUnifiedPush::AbstractPushProvider::messageAcknowledged(const KUnifiedPush::Client & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void KUnifiedPush::AbstractPushProvider::urgencyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
