/****************************************************************************
** Meta object code from reading C++ file 'connector.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kunifiedpush-26.04.3/src/client/connector.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'connector.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KUnifiedPush9ConnectorE_t {};
} // unnamed namespace

template <> constexpr inline auto KUnifiedPush::Connector::qt_create_metaobjectdata<qt_meta_tag_ZN12KUnifiedPush9ConnectorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KUnifiedPush::Connector",
        "messageReceived",
        "",
        "msg",
        "endpointChanged",
        "endpoint",
        "stateChanged",
        "KUnifiedPush::Connector::State",
        "state",
        "vapidPublicKeyChanged",
        "vapidPublicKeyRequiredChanged",
        "State",
        "vapidPublicKey",
        "vapidPublicKeyRequired",
        "Unregistered",
        "Registering",
        "Registered",
        "NoDistributor",
        "Error"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'messageReceived'
        QtMocHelpers::SignalData<void(const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 },
        }}),
        // Signal 'endpointChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(KUnifiedPush::Connector::State)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'vapidPublicKeyChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'vapidPublicKeyRequiredChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'endpoint'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
        // property 'state'
        QtMocHelpers::PropertyData<enum State>(8, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'vapidPublicKey'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'vapidPublicKeyRequired'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(11, 11, QMC::EnumFlags{}).add({
            {   14, State::Unregistered },
            {   15, State::Registering },
            {   16, State::Registered },
            {   17, State::NoDistributor },
            {   18, State::Error },
        }),
    };
    return QtMocHelpers::metaObjectData<Connector, qt_meta_tag_ZN12KUnifiedPush9ConnectorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KUnifiedPush::Connector::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KUnifiedPush9ConnectorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KUnifiedPush9ConnectorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KUnifiedPush9ConnectorE_t>.metaTypes,
    nullptr
} };

void KUnifiedPush::Connector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Connector *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->messageReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 1: _t->endpointChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<KUnifiedPush::Connector::State>>(_a[1]))); break;
        case 3: _t->vapidPublicKeyChanged(); break;
        case 4: _t->vapidPublicKeyRequiredChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Connector::*)(const QByteArray & )>(_a, &Connector::messageReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Connector::*)(const QString & )>(_a, &Connector::endpointChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Connector::*)(KUnifiedPush::Connector::State )>(_a, &Connector::stateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Connector::*)()>(_a, &Connector::vapidPublicKeyChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Connector::*)()>(_a, &Connector::vapidPublicKeyRequiredChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->endpoint(); break;
        case 1: *reinterpret_cast<enum State*>(_v) = _t->state(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->vapidPublicKey(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->vapidPublicKeyRequired(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setVapidPublicKey(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setVapidPublicKeyRequired(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KUnifiedPush::Connector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KUnifiedPush::Connector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KUnifiedPush9ConnectorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KUnifiedPush::Connector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void KUnifiedPush::Connector::messageReceived(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KUnifiedPush::Connector::endpointChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KUnifiedPush::Connector::stateChanged(KUnifiedPush::Connector::State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KUnifiedPush::Connector::vapidPublicKeyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KUnifiedPush::Connector::vapidPublicKeyRequiredChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
