/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kunifiedpush.distributor.Management.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef MANAGEMENTADAPTOR_H
#define MANAGEMENTADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "distributor.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.kunifiedpush.Management
 */
class ManagementAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.kunifiedpush.Management")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.kunifiedpush.Management\">\n"
"    <method name=\"status\">\n"
"      <arg direction=\"out\" type=\"i\" name=\"status\"/>\n"
"    </method>\n"
"    <signal name=\"statusChanged\"/>\n"
"    <method name=\"errorMessage\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"errorMessage\"/>\n"
"    </method>\n"
"    <signal name=\"errorMessageChanged\"/>\n"
"    <method name=\"pushProviderId\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"status\"/>\n"
"    </method>\n"
"    <method name=\"pushProviderConfiguration\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"pushProviderId\"/>\n"
"      <arg direction=\"out\" type=\"a{sv}\" name=\"config\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"setPushProvider\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"pushProvider\"/>\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"config\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In1\"/>\n"
"      <annotation value=\"true\" name=\"org.freedesktop.DBus.Method.NoReply\"/>\n"
"    </method>\n"
"    <signal name=\"pushProviderChanged\"/>\n"
"    <method name=\"registeredClients\">\n"
"      <arg direction=\"out\" type=\"a(sss)\" name=\"clients\"/>\n"
"      <annotation value=\"QList&lt;KUnifiedPush::ClientInfo&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"forceUnregisterClient\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"clientToken\"/>\n"
"      <annotation value=\"true\" name=\"org.freedesktop.DBus.Method.NoReply\"/>\n"
"    </method>\n"
"    <signal name=\"registeredClientsChanged\"/>\n"
"  </interface>\n"
        "")
public:
    ManagementAdaptor(KUnifiedPush::Distributor *parent);
    ~ManagementAdaptor() override;

    inline KUnifiedPush::Distributor *parent() const
    { return static_cast<KUnifiedPush::Distributor *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QString errorMessage();
    Q_NOREPLY void forceUnregisterClient(const QString &clientToken);
    QVariantMap pushProviderConfiguration(const QString &pushProviderId);
    QString pushProviderId();
    QList<KUnifiedPush::ClientInfo> registeredClients();
    Q_NOREPLY void setPushProvider(const QString &pushProvider, const QVariantMap &config);
    int status();
Q_SIGNALS: // SIGNALS
    void errorMessageChanged();
    void pushProviderChanged();
    void registeredClientsChanged();
    void statusChanged();
};

#endif
