/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kunifiedpush.distributor.Management.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef MANAGEMENTINTERFACE_H
#define MANAGEMENTINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "../shared/clientinfo_p.h"

/*
 * Proxy class for interface org.kde.kunifiedpush.Management
 */
class OrgKdeKunifiedpushManagementInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kunifiedpush.Management"; }

public:
    OrgKdeKunifiedpushManagementInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKunifiedpushManagementInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QString> errorMessage()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("errorMessage"), argumentList);
    }

    Q_NOREPLY inline void forceUnregisterClient(const QString &clientToken)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(clientToken);
        callWithArgumentList(QDBus::NoBlock, QStringLiteral("forceUnregisterClient"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> pushProviderConfiguration(const QString &pushProviderId)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(pushProviderId);
        return asyncCallWithArgumentList(QStringLiteral("pushProviderConfiguration"), argumentList);
    }

    inline QDBusPendingReply<QString> pushProviderId()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("pushProviderId"), argumentList);
    }

    inline QDBusPendingReply<QList<KUnifiedPush::ClientInfo> > registeredClients()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("registeredClients"), argumentList);
    }

    Q_NOREPLY inline void setPushProvider(const QString &pushProvider, const QVariantMap &config)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(pushProvider) << QVariant::fromValue(config);
        callWithArgumentList(QDBus::NoBlock, QStringLiteral("setPushProvider"), argumentList);
    }

    inline QDBusPendingReply<int> status()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("status"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void errorMessageChanged();
    void pushProviderChanged();
    void registeredClientsChanged();
    void statusChanged();
};

namespace org {
  namespace kde {
    namespace kunifiedpush {
      using Management = ::OrgKdeKunifiedpushManagementInterface;
    }
  }
}

#endif
