/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.unifiedpush.Distributor1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef DISTRIBUTOR1ADAPTOR_H
#define DISTRIBUTOR1ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "distributor.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.unifiedpush.Distributor1
 */
class Distributor1Adaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.unifiedpush.Distributor1")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.unifiedpush.Distributor1\">\n"
"    <method name=\"Register\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"serviceName\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"token\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"description\"/>\n"
"      <arg direction=\"out\" type=\"s\" name=\"registrationResult\"/>\n"
"      <arg direction=\"out\" type=\"s\" name=\"registrationResultReason\"/>\n"
"    </method>\n"
"    <method name=\"Unregister\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"token\"/>\n"
"      <annotation value=\"true\" name=\"org.freedesktop.DBus.Method.NoReply\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    Distributor1Adaptor(KUnifiedPush::Distributor *parent);
    ~Distributor1Adaptor() override;

    inline KUnifiedPush::Distributor *parent() const
    { return static_cast<KUnifiedPush::Distributor *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QString Register(const QString &serviceName, const QString &token, const QString &description, QString &registrationResultReason);
    Q_NOREPLY void Unregister(const QString &token);
Q_SIGNALS: // SIGNALS
};

#endif
