/****************************************************************************
** Meta object code from reading C++ file 'kcmpushnotifications.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kunifiedpush-26.04.3/src/kcm/kcmpushnotifications.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kcmpushnotifications.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20KCMPushNotificationsE_t {};
} // unnamed namespace

template <> constexpr inline auto KCMPushNotifications::qt_create_metaobjectdata<qt_meta_tag_ZN20KCMPushNotificationsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCMPushNotifications",
        "distributorChanged",
        "",
        "distributorStatusChanged",
        "distributorErrorMessageChanged",
        "pushProviderChanged",
        "nextcloudAuthenticated",
        "loginName",
        "appPassword",
        "saveRequested",
        "forceUnregister",
        "token",
        "setPushProviderConfiguration",
        "pushProviderId",
        "QVariantMap",
        "config",
        "pushProviderConfiguration",
        "nextcloudAuthenticate",
        "QUrl",
        "url",
        "hasDistributor",
        "hasKDEDistributor",
        "distributorStatus",
        "distributorErrorMessage",
        "clientModel",
        "ClientModel*",
        "selfTest",
        "SelfTest*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'distributorChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'distributorStatusChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'distributorErrorMessageChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pushProviderChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nextcloudAuthenticated'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'saveRequested'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'forceUnregister'
        QtMocHelpers::SlotData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'setPushProviderConfiguration'
        QtMocHelpers::SlotData<void(const QString &, const QVariantMap &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Method 'pushProviderConfiguration'
        QtMocHelpers::MethodData<QVariantMap(const QString &) const>(16, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 13 },
        }}),
        // Method 'nextcloudAuthenticate'
        QtMocHelpers::MethodData<void(const QUrl &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'hasDistributor'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'hasKDEDistributor'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'distributorStatus'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'distributorErrorMessage'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'pushProviderId'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'clientModel'
        QtMocHelpers::PropertyData<ClientModel*>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'selfTest'
        QtMocHelpers::PropertyData<SelfTest*>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KCMPushNotifications, qt_meta_tag_ZN20KCMPushNotificationsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCMPushNotifications::staticMetaObject = { {
    QMetaObject::SuperData::link<KQuickConfigModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KCMPushNotificationsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KCMPushNotificationsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20KCMPushNotificationsE_t>.metaTypes,
    nullptr
} };

void KCMPushNotifications::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KCMPushNotifications *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->distributorChanged(); break;
        case 1: _t->distributorStatusChanged(); break;
        case 2: _t->distributorErrorMessageChanged(); break;
        case 3: _t->pushProviderChanged(); break;
        case 4: _t->nextcloudAuthenticated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->saveRequested(); break;
        case 6: _t->forceUnregister((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->setPushProviderConfiguration((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 8: { QVariantMap _r = _t->pushProviderConfiguration((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->nextcloudAuthenticate((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KCMPushNotifications::*)()>(_a, &KCMPushNotifications::distributorChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCMPushNotifications::*)()>(_a, &KCMPushNotifications::distributorStatusChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCMPushNotifications::*)()>(_a, &KCMPushNotifications::distributorErrorMessageChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCMPushNotifications::*)()>(_a, &KCMPushNotifications::pushProviderChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCMPushNotifications::*)(const QString & , const QString & )>(_a, &KCMPushNotifications::nextcloudAuthenticated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCMPushNotifications::*)()>(_a, &KCMPushNotifications::saveRequested, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ClientModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->hasDistributor(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->hasKDEDistributor(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->distributorStatus(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->distributorErrorMessage(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->pushProviderId(); break;
        case 5: *reinterpret_cast<ClientModel**>(_v) = _t->clientModel(); break;
        case 6: *reinterpret_cast<SelfTest**>(_v) = _t->selfTest(); break;
        default: break;
        }
    }
}

const QMetaObject *KCMPushNotifications::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCMPushNotifications::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KCMPushNotificationsE_t>.strings))
        return static_cast<void*>(this);
    return KQuickConfigModule::qt_metacast(_clname);
}

int KCMPushNotifications::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KQuickConfigModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KCMPushNotifications::distributorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KCMPushNotifications::distributorStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KCMPushNotifications::distributorErrorMessageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KCMPushNotifications::pushProviderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KCMPushNotifications::nextcloudAuthenticated(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void KCMPushNotifications::saveRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
