/****************************************************************************
** Meta object code from reading C++ file 'distributorstatus_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kunifiedpush-26.04.3/src/shared/distributorstatus_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'distributorstatus_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KUnifiedPush17DistributorStatusE_t {};
} // unnamed namespace

template <> constexpr inline auto KUnifiedPush::DistributorStatus::qt_create_metaobjectdata<qt_meta_tag_ZN12KUnifiedPush17DistributorStatusE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KUnifiedPush::DistributorStatus",
        "Status",
        "Unknown",
        "Idle",
        "Connected",
        "NoNetwork",
        "AuthenticationError",
        "NoSetup"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<Status>(1, 1, QMC::EnumFlags{}).add({
            {    2, Status::Unknown },
            {    3, Status::Idle },
            {    4, Status::Connected },
            {    5, Status::NoNetwork },
            {    6, Status::AuthenticationError },
            {    7, Status::NoSetup },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN12KUnifiedPush17DistributorStatusE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN12KUnifiedPush17DistributorStatusE =
    KUnifiedPush::DistributorStatus::qt_create_metaobjectdata<qt_meta_tag_ZN12KUnifiedPush17DistributorStatusE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN12KUnifiedPush17DistributorStatusE =
    qt_staticMetaObjectContent_ZN12KUnifiedPush17DistributorStatusE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN12KUnifiedPush17DistributorStatusE =
    qt_staticMetaObjectContent_ZN12KUnifiedPush17DistributorStatusE.relocatingData;

Q_CONSTINIT const QMetaObject KUnifiedPush::DistributorStatus::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN12KUnifiedPush17DistributorStatusE.stringdata,
    qt_staticMetaObjectStaticContent_ZN12KUnifiedPush17DistributorStatusE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN12KUnifiedPush17DistributorStatusE.metaTypes,
    nullptr
} };

QT_WARNING_POP
