/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.unifiedpush.Distributor1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef DISTRIBUTOR1IFACE_H
#define DISTRIBUTOR1IFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.unifiedpush.Distributor1
 */
class OrgUnifiedpushDistributor1Interface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.unifiedpush.Distributor1"; }

public:
    OrgUnifiedpushDistributor1Interface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgUnifiedpushDistributor1Interface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QString, QString> Register(const QString &serviceName, const QString &token, const QString &description)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(serviceName) << QVariant::fromValue(token) << QVariant::fromValue(description);
        return asyncCallWithArgumentList(QStringLiteral("Register"), argumentList);
    }
    inline QDBusReply<QString> Register(const QString &serviceName, const QString &token, const QString &description, QString &registrationResultReason)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(serviceName) << QVariant::fromValue(token) << QVariant::fromValue(description);
        QDBusMessage reply = callWithArgumentList(QDBus::Block, QStringLiteral("Register"), argumentList);
        if (reply.type() == QDBusMessage::ReplyMessage && reply.arguments().size() == 2) {
            registrationResultReason = qdbus_cast<QString>(reply.arguments().at(1));
        }
        return reply;
    }

    Q_NOREPLY inline void Unregister(const QString &token)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(token);
        callWithArgumentList(QDBus::NoBlock, QStringLiteral("Unregister"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace unifiedpush {
    using Distributor1 = ::OrgUnifiedpushDistributor1Interface;
  }
}

#endif
